;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; ARROW2 v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure draws filled, thick arrows. It uses DEVICE coordinates by     ;
;  default.                                                                    ;
;                                                                              ;
; Positional parameters:                                                       ;
; x0, y0:   Coordinates of the origin.                                         ;
; x1, y1:   Coordinates of the endpoint.                                       ;
;                                                                              ;
; Keyword parameters:                                                          ;
; LA:       Arrowhead length. By default, it is 0.15 times the arrow length.   ;
; DX1:      Arrow thickness. By default, it is 0.20 times the arrowhead length.;
; DX2:      Extra arrowhead thickness. By default, it is equal to 2xdx1.       ;
; ANGLEOUT: Angle in CCW degrees from the top for the arrow (output).          ;
; COLTEXT:  Arrow color.                                                       ;
; DEVICE:   Flag to use DEVICE (default, so it is obsolete)                    ;
; NORMAL:   Flag to use NORMAL.                                                ;
; DATA:     Flag to use DATA.                                                  ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:     A bug with DATA coordinates is fixed.                              ;
;           ANGLEOUT is added.                                                 ;
; v0.2:     LA, DX, AND DX2 are changed from positional to keyword parameters. ;
;           DX is changed to DX1.                                              ;
;           Bugs with DATA and NORMAL coordinates are fixed.                   ;
; v1.0:     General cleanup.                                                   ;
; v2.0:     English version.                                                   ;
; v2.1:     IDLdoc documentation.                                              ;
; v2.5:     Version system update.                                             ;
; v3.0:     Adapted to v3.0 for SINGLEPLOT.                                    ;
;           Negative values for LA, DX1, and DX2 are assumed to be default.    ;
; v3.1:     Only version change.                                               ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO ARROW2, x0, y0, x1, y1, LA=la, DX1=dx1, DX2=dx2, ANGLEOUT=angleout, $
               COLTEXT=coltext, DEVICE=device, NORMAL=normal, DATA=data
ON_ERROR, 1
;
; Valores originales
;
x0orig = x0 & y0orig = y0 & x1orig = x1 & y1orig = y1
IF N_ELEMENTS(LA)  NE 0 THEN laorig  = la  ELSE laorig  = -99.99
IF N_ELEMENTS(DX1) NE 0 THEN dx1orig = dx1 ELSE dx1orig = -99.99
IF N_ELEMENTS(DX2) NE 0 THEN dx2orig = dx2 ELSE dx2orig = -99.99
;
; Eleccin de tipo de coordenadas
;
device = 1
fac    = 1.0
IF KEYWORD_SET(NORMAL)*KEYWORD_SET(DATA) NE 0 THEN MESSAGE, $
  'Error in coordinate type'
IF KEYWORD_SET(NORMAL) THEN BEGIN
;    device = 0 & normal = 1
 device = 1 & normal = 0
 aux1 = [[x0,y0],[x1,y1]]
 aux2 = CONVERT_COORD(aux1, /NORMAL, /TO_DEVICE)
 x0 = aux2[0,0] & x1 = aux2[0,1]
 y0 = aux2[1,0] & y1 = aux2[1,1]
 aux3 = CONVERT_COORD([[0.0,0.0],[1.0,0.0]], /NORMAL, /TO_DEVICE)
 fac  = aux3[0,1]-aux3[0,0]
ENDIF ELSE BEGIN 
 normal = 0
ENDELSE
IF KEYWORD_SET(DATA) THEN BEGIN
 device = 1 & normal = 0
 aux1 = [[x0,y0],[x1,y1]]
 aux2 = CONVERT_COORD(aux1, /DATA, /TO_DEVICE)
 x0 = aux2[0,0] & x1 = aux2[0,1]
 y0 = aux2[1,0] & y1 = aux2[1,1]
 aux3 = CONVERT_COORD([[!X.RANGE[0],!Y.RANGE[0]],[!X.RANGE[1],!Y.RANGE[1]]], $
                      /DATA, /TO_DEVICE)
 fac  = ABS((aux3[0,1]-aux3[0,0])/(!X.RANGE[1]-!X.RANGE[0]))
; IF x1 NE x0 THEN fac  = ABS((aux3[0,1]-aux3[0,0])/(!X.RANGE[1]-!X.RANGE[0])) $
 ;            ELSE fac  = ABS((aux3[1,1]-aux3[1,0])/(!Y.RANGE[1]-!Y.RANGE[0]))
ENDIF
;
; Inicializacin y asignacin de variables
; 
IF N_ELEMENTS(COLTEXT) EQ 0 THEN coltext = 0
l  = SQRT((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0))
theta = ATAN((y1-y0),(x1-x0))
angleout = 180*theta/!PI
IF laorig  LE 0.0 THEN la  = 0.15*l  ELSE la  =  la*fac
IF dx1orig LE 0.0 THEN dx1 = 0.20*la ELSE dx1 = dx1*fac
IF dx2orig LE 0.0 THEN dx2 = 0.40*la ELSE dx2 = dx2*fac
IF la  EQ 0 THEN la  = 0.15*l
IF dx1 EQ 0 THEN dx1 = 0.20*la
IF dx2 EQ 0 THEN dx2 = dx1  
l1 = 0.5*dx1*COS(theta)
l2 = 0.5*dx1*SIN(theta)
l3 = (l-la)*COS(theta)
l4 = (l-la)*SIN(theta)
l5 = l*COS(theta)
l6 = l*SIN(theta)
l7 = dx2*COS(theta)
l8 = dx2*SIN(theta)
;
; Generacin del polinomio
; 
x = [x0+l2,x0+l2+l3,x0+l2+l8+l3,x0+l5,x0-l2-l8+l3,x0-l2+l3,x0-l2,x0+l2]
y = [y0-l1,y0-l1+l4,y0-l1-l7+l4,y0+l6,y0+l1+l7+l4,y0+l1+l4,y0+l1,y0-l1]
;
; Dibujo de la flecha
; 
POLYFILL, x, y, COLOR=coltext, DEVICE=device, NORMAL=normal
;
; Restauracin de valores originales
;
x0 = x0orig & y0  = y0orig  & x1  = x1orig  & y1 = y1orig
la = laorig & dx1 = dx1orig & dx2 = dx2orig
END

