;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; AXIS2 v. 3.1, 11 May 2013.                                                   ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure draws axes.                                                   ;
;                                                                              ;
; Positional parameters:                                                       ;
; pplot:    Plot structure.                                                    ;
; axis:     Axis structure to be plotted.                                      ;
; axis2:    Perpendicular lower axis structure.                                ;
; cfile:    Color file structure.                                              ;
;                                                                              ;
; Keyword parameters:                                                          ;
; V:        Value to be used for axis positioning.                             ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:     The call to FTICKS from here is made optional.                     ;
;           NOTEXT keyword added.                                              ;
;           PSASPECT keyword added.                                            ;
;           CTEXT introduced.                                                  ;
; v0.2:     EXTRASEPAXIS added.                                                ;
; v1.0:     General cleanup.                                                   ;
; v1.1:     NOTICKS added.                                                     ;
; v1.2:     FONT added.                                                        ;
; v2.0:     English version.                                                   ;
; v2.1:     IDLdoc documentation.                                              ;
; v2.5:     Version system update.                                             ;
; v3.0:     Overhaul due to SINGLEPLOT and SINGLECONTOUR changes.              ;
; v3.1:     Only version change.                                               ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO AXIS2, pplot, axis, axis2, cfile, V=v, SPANISH=spanish
ON_ERROR, 1
;
; Inicializacin
;
extrasepxaxis  = 1.0
extrasepxaxis2 = 1.0 + 0.5*(extrasepxaxis - 1.0)
typexy = STRMID(axis.type,0,1)
typelu = STRMID(axis.type,1,1)
IF N_ELEMENTS(v) EQ 0 THEN BEGIN
 IF typelu EQ 'l' THEN v = axis2.range[0] $
                  ELSE v = axis2.range[1]
ENDIF
sign   = -1*(typelu EQ 'l') +1*(typelu EQ 'u')
;IF !X.MINOR EQ 0 THEN !X.MINOR = 5
;IF !Y.MINOR EQ 0 THEN !Y.MINOR = 5
;
; Dibujamos el eje
; 
IF typexy EQ 'x' THEN PLOTS, axis.range, [v,v], THICK=axis.thick, LINESTYLE=axis.linestyle, /DATA, COLOR=cfile.def $
                 ELSE PLOTS, [v,v], axis.range, THICK=axis.thick, LINESTYLE=axis.linestyle, /DATA, COLOR=cfile.def
;
; Dibujamos las marcas y su texto
; 
IF typexy EQ 'x' THEN BEGIN
; Marcas mayores y menores del eje x
 IF axis.style GE 0 THEN BEGIN
  vn      = CONVERT_COORD(axis.tickv[0], v             , /DATA, /TO_NORMAL)
  length  = (!P.POSITION[2]-!P.POSITION[0])*(-sign)*0.04*pplot.ysize*(pplot.psxsize/19.0)*axis.ticklen
  xtickn  = CONVERT_COORD(axis.tickv   , axis2.tickv[0], /DATA, /TO_NORMAL)
  FOR i=0,N_ELEMENTS(axis.tickv)-1 DO $
   PLOTS,  [xtickn[0,i],xtickn[0,i]]  , [vn[1,0],vn[1,0]+length]    , $
           THICK=axis.thick, LINESTYLE=axis.linestyle, /NORMAL, COLOR=cfile.def
  IF axis.grid NE 0 THEN FOR i=0,N_ELEMENTS(axis.tickv)-1 DO $
   PLOTS,  [axis.tickv[i],axis.tickv[i]], axis2.range, LINESTYLE=1, /DATA, COLOR=cfile.def
  vminorn = CONVERT_COORD(axis.minorv  , axis2.tickv[0], /DATA, /TO_NORMAL)
  FOR i=0,N_ELEMENTS(axis.minorv)-1 DO $
   IF vminorn[0,i] GE !P.POSITION[0] AND vminorn[0,i] LE !P.POSITION[2] THEN $
    PLOTS, [vminorn[0,i],vminorn[0,i]], [vn[1,0],vn[1,0]+0.5*length], $
           THICK=axis.thick, LINESTYLE=axis.linestyle, /NORMAL, COLOR=cfile.def
 ENDIF
; Texto de las marcas principales y ttulo del eje x
 IF axis.style EQ 1 THEN BEGIN
  dy      = 0.15*pplot.unitl/pplot.psaspect*sign + $
            0.25*pplot.unitl/pplot.psaspect*0.5*(sign-1)*extrasepxaxis
  XYOUTS, xtickn[0,*], vn[1,0]+dy, axis.tickname2, $
   ALIGNMENT=0.5, /NORMAL, CHARTHICK=pplot.charthick, $
   CHARSIZE=pplot.charsize*axis.charsize, COLOR=cfile.def, FONT=pplot.font
  IF axis.title NE '' THEN BEGIN
   IF typelu EQ 'l' THEN $
    XYOUTS, 0.5*(!P.POSITION(0)+!P.POSITION(2)), $
            !P.POSITION(1)-1.00*pplot.unitl/pplot.psaspect*extrasepxaxis2, $
	    axis.title, $
            COLOR=cfile.def, FONT=pplot.font, $
	    CHARTHICK=pplot.charthick, CHARSIZE=pplot.charsize*axis.charsize, ALIGNMENT=0.5, /NORMAL ELSE $
    XYOUTS, 0.5*(!P.POSITION(0)+!P.POSITION(2)), $
            !P.POSITION(3)+0.80*pplot.unitl/pplot.psaspect*extrasepxaxis2, $
	    axis.title, $
            COLOR=cfile.def, FONT=pplot.font, $
	    CHARTHICK=pplot.charthick, CHARSIZE=pplot.charsize*axis.charsize, ALIGNMENT=0.5, /NORMAL
  ENDIF
 ENDIF
ENDIF ELSE BEGIN
; Marcas mayores y menores del eje y
 IF axis.style GE 0 THEN BEGIN
  vn      = CONVERT_COORD(v             , axis.tickv[0], /DATA, /TO_NORMAL)
  length  = (!P.POSITION[3]-!P.POSITION[1])*(-sign)*0.04*pplot.xsize*(pplot.psysize/19.0)*axis.ticklen
  ytickn  = CONVERT_COORD(axis2.tickv[0], axis.tickv   , /DATA, /TO_NORMAL)
  FOR i=0,N_ELEMENTS(axis.tickv)-1 DO $
   PLOTS,  [vn[0,0],vn[0,0]+length]    , [ytickn[1,i],ytickn[1,i]]  , $
           THICK=axis.thick, LINESTYLE=axis.linestyle, /NORMAL, COLOR=cfile.def
  IF axis.grid NE 0 THEN FOR i=0,N_ELEMENTS(axis.tickv)-1 DO $
   PLOTS,  axis2.range, [axis.tickv[i],axis.tickv[i]], LINESTYLE=1, /DATA, COLOR=cfile.def
  vminorn = CONVERT_COORD(axis2.tickv[0], axis.minorv  , /DATA, /TO_NORMAL)
  FOR i=0,N_ELEMENTS(axis.minorv)-1 DO $
   IF vminorn[1,i] GE !P.POSITION[1] AND vminorn[1,i] LE !P.POSITION[3] THEN $
    PLOTS, [vn[0,0],vn[0,0]+0.5*length], [vminorn[1,i],vminorn[1,i]], $
           THICK=axis.thick, LINESTYLE=axis.linestyle, /NORMAL, COLOR=cfile.def
 ENDIF
; Texto de las marcas principales y ttulo del eje y
 IF axis.style EQ 1 THEN BEGIN
  dx      =  0.15*pplot.unitl*sign
  dy      = -0.10*pplot.unitl/pplot.psaspect
  XYOUTS, vn[0,0]+dx, ytickn[1,*]+dy, axis.tickname2, $
   ALIGNMENT=-0.5*(sign-1), /NORMAL, CHARTHICK=pplot.charthick, $
   CHARSIZE=pplot.charsize*axis.charsize, COLOR=cfile.def, FONT=pplot.font
  IF axis.title NE '' THEN BEGIN
   IF typelu EQ 'l' THEN $
    XYOUTS, !P.POSITION(0)-pplot.extraxl+0.40*pplot.unitl, $
            0.5*(!P.POSITION(1)+!P.POSITION(3)), $
	    axis.title, $
            COLOR=cfile.def, FONT=pplot.font, ORIENTATION=90 , $
	    CHARTHICK=pplot.charthick, CHARSIZE=pplot.charsize*axis.charsize, ALIGNMENT=0.5, /NORMAL ELSE $
    XYOUTS, !P.POSITION(2)+pplot.extraxu-0.40*pplot.unitl, $
            0.5*(!P.POSITION(1)+!P.POSITION(3)), $
	    axis.title, $
            COLOR=cfile.def, FONT=pplot.font, ORIENTATION=270, $
	    CHARTHICK=pplot.charthick, CHARSIZE=pplot.charsize*axis.charsize, ALIGNMENT=0.5, /NORMAL
  ENDIF
 ENDIF
ENDELSE
END
