;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; COMMONSINGLE v. 3.1, 11 May 2013.                                            ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure does the initialization for SINGLEPLOT and SINGLECONTOUR.     ;
;                                                                              ;
; Positional parameters:                                                       ;
; xxl:        Output lower x axis structure.                                   ;
; yyl:        Output lower y axis structure.                                   ;
; xxu:        Output upper x axis structure.                                   ;
; yyu:        Output upper y axis structure.                                   ;
; pplot:      Output plot structure.                                           ;
; nplots:     Number of data structures.                                       ;
; v1:         Output dummy data structure (BACKGROUND, no data).               ;
;                                                                              ;
; Keyword parameters:                                                          ;
; BACKGROUND: Background image of dimensions (nx,ny) or (nx,ny,3).             ;
; XAXISL:     Input lower x axis structure.                                    ;
; YAXISL:     Input lower y axis structure.                                    ;
; XAXISU:     Input upper x axis structure.                                    ;
; YAXISU:     Input upper y axis structure.                                    ;
; PLOT:       Input plot structure.                                            ;
; CFILE:      Input/output color file structure.                               ;
; LEGEND:     Input/output legend structure.                                   ;
; QUIET:      Flag to supress informational messages.                          ;
; IN:         Input structure to pass the structures produced by out.          ;
; DAXIS:      [yleft, xdown, yright, xup] array with options for the four axes.;
;              Allowed values are: -2 (nothing), -1 (lines), 0 (lines and      ;
;              ticks), and 1 (lines, ticks, and text).                         ;
; MINMAX:     [xmin,ymin,xmax,ymax] values for the x and y ranges.             ;
; XRANGE:     [xmin,xmax] values for the x range.                              ;
; YRANGE:     [ymin,ymax] values for the y range.                              ;
; PSASPECT:   Aspect ratio of the Postscript file.                             ;
; PSXSIZE:    X size (in cm) of the Postscript file.                           ;
; LANDSCAPE:  Flag for landscape mode.                                         ;
; PSFONTS:    Flag to use Postscript fonts.                                    ;
;                                                                              ;
; Version history:                                                             ;
; v3.0        First version.                                                   ;
; v3.1        NOBORDER added to LEGEND.                                        ;
;             Default style of opposite axes changed to 0 (from 1).            ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO COMMONSINGLE, xxl, yyl, xxu, yyu, pplot, nplots, v1, $
	          CFILE=cfile, LEGEND=legend, BACKGROUND=background, $
		  XAXISL=xaxisl, YAXISL=yaxisl, $
		  XAXISU=xaxisu, YAXISU=yaxisu, $
		  PLOT=plot, QUIET=quiet, IN=in,   $
		  DAXIS=daxis, MINMAX=minmax, XRANGE=xrange, YRANGE=yrange, $
		  PSASPECT=psaspect, PSXSIZE=psxsize, $
		  LANDSCAPE=landscape, PSFONTS=psfonts
;
; Primera inicializacin de ejes (XL, YL, XU, YU), PLOT, CFILE y LEGEND
; style siempre se inicializa par los ejes y psaspect siempre se inicializa para PLOT
; range solamente se inicializa para los ejes si existe BACKGROUND
; name se inicializa siempre para cfile
; draw y text se inicializan siempre para legend
;
IF KEYWORD_SET(IN) THEN BEGIN
 IF TAG_EXIST(in,'xaxisl') THEN xaxisl = in.xaxisl
 IF TAG_EXIST(in,'yaxisl') THEN yaxisl = in.yaxisl
 IF TAG_EXIST(in,'xaxisu') THEN xaxisu = in.xaxisu
 IF TAG_EXIST(in,'yaxisu') THEN yaxisu = in.yaxisu
 IF TAG_EXIST(in,'plot')   THEN plot   = in.plot
 IF TAG_EXIST(in,'cfile')  THEN cfile  = in.cfile
 IF TAG_EXIST(in,'legend') THEN legend = in.legend
ENDIF
IF  KEYWORD_SET(MINMAX) NE 0 AND KEYWORD_SET(QUIET) EQ 0 THEN $
    PRINT, 'Warning: using MINMAX is obsolete!'
IF (KEYWORD_SET(XRANGE) NE 0 OR $
    KEYWORD_SET(YRANGE) NE 0) AND KEYWORD_SET(QUIET) EQ 0 THEN $
    PRINT, 'Warning: using XRANGE/YRANGE is obsolete!'
IF  KEYWORD_SET(DAXIS)  NE 0 AND KEYWORD_SET(QUIET) EQ 0 THEN $
 PRINT, 'Warning: using DAXIS is obsolete!'
IF  KEYWORD_SET(PSASPECT)+KEYWORD_SET(PSXSIZE)+KEYWORD_SET(PSFONTS)+KEYWORD_SET(LANDSCAPE) NE 0 AND $
    KEYWORD_SET(QUIET)  EQ 0 THEN PRINT, 'Warning: using obsolete PS keywords!'
IF  KEYWORD_SET(MINMAX) NE 0 THEN BEGIN ; Combinacin de casos de SINGLEPLOT y SINGLECONTOUR
 xrange  = [minmax[0],minmax[2]]
 yrange  = [minmax[1],minmax[3]]
ENDIF
IF N_ELEMENTS(XAXISL)   NE 0 THEN xxl   = xaxisl ELSE xxl   = {dummy:0}
IF N_ELEMENTS(YAXISL)   NE 0 THEN yyl   = yaxisl ELSE yyl   = {dummy:0}
IF N_ELEMENTS(XAXISU)   NE 0 THEN xxu   = xaxisu ELSE xxu   = xxl
IF N_ELEMENTS(YAXISU)   NE 0 THEN yyu   = yaxisu ELSE yyu   = yyl
IF N_ELEMENTS(PLOT)     NE 0 THEN pplot = plot   ELSE pplot = {dummy:0}
IF N_ELEMENTS(CFILE)    EQ 0 THEN BEGIN
 cfile  = {name:'colorjma.tbl'}
ENDIF ELSE IF SIZE(cfile,/TNAME) NE 'STRUCT' THEN BEGIN
 IF KEYWORD_SET(QUIET)  EQ 0 THEN PRINT, 'Using obsolete CFILE keywords!'
 cfile  = {name:cfile}
ENDIF
IF N_ELEMENTS(LEGEND)   EQ 0 THEN BEGIN
 legend = {draw:0, text:''}
ENDIF ELSE IF SIZE(legend,/TNAME) NE 'STRUCT' THEN BEGIN
 IF KEYWORD_SET(QUIET)  EQ 0 THEN PRINT, 'Using obsolete LEGEND keywords!'
 legend = {draw:1, text:legend}
ENDIF ELSE BEGIN
 IF TAG_EXIST(legend,'text') EQ 0 THEN legend = CREATE_STRUCT(legend, {text:''})
 IF TAG_EXIST(legend,'draw') EQ 0 THEN $
  IF MAX(STRLEN(legend.text)) NE 0 OR   $
     TAG_EXIST(legend,'xsize') + TAG_EXIST(legend,'ysize') + $
     TAG_EXIST(legend,'xleft') + TAG_EXIST(legend,'ydown') + $
     TAG_EXIST(legend,'clear') GE 1 THEN legend = CREATE_STRUCT(legend, {draw:1})
 IF TAG_EXIST(legend,'noborder') EQ 0 THEN legend = CREATE_STRUCT(legend, {noborder:0})
ENDELSE
IF SIZE(xxl  ,/TNAME)  NE 'STRUCT' THEN MESSAGE, 'XAXISL has to be a structure'
IF SIZE(yyl  ,/TNAME)  NE 'STRUCT' THEN MESSAGE, 'YAXISL has to be a structure'
IF SIZE(xxu  ,/TNAME)  NE 'STRUCT' THEN MESSAGE, 'XAXISU has to be a structure'
IF SIZE(yyu  ,/TNAME)  NE 'STRUCT' THEN MESSAGE, 'YAXISU has to be a structure'
IF SIZE(pplot,/TNAME)  NE 'STRUCT' THEN MESSAGE, 'PLOT   has to be a structure'
IF KEYWORD_SET(BACKGROUND) THEN BEGIN ; Preparacin de aspecto PS para el caso en que hay fondo
 sizeback = SIZE(background,/DIM)
 npixx    = sizeback[0]
 npixy    = sizeback[1]
 IF TAG_EXIST(pplot,'psaspect') EQ 0 THEN $
  IF KEYWORD_SET(PSASPECT)      EQ 0 THEN pplot = CREATE_STRUCT(pplot, {psaspect:FLOAT(npixy)/npixx}) $
                                     ELSE pplot = CREATE_STRUCT(pplot, {psaspect:psaspect          })
ENDIF ELSE BEGIN ; Si no hay fondo eso se hace despus
 IF TAG_EXIST(pplot,'psaspect') EQ 0 THEN $
  IF KEYWORD_SET(PSASPECT)      EQ 0 THEN pplot = CREATE_STRUCT(pplot, {psaspect:               0.0}) $
                                     ELSE pplot = CREATE_STRUCT(pplot, {psaspect:psaspect          })
ENDELSE
IF N_ELEMENTS(pplot.psaspect)  NE 1 THEN MESSAGE, 'Incorrect dimensions for plot.psaspect'
IF SIZE(pplot.psaspect,/TYPE)  NE 4 AND $
   SIZE(pplot.psaspect,/TYPE)  NE 5 THEN COPY_STRUC, pplot, pplot, EXCEPT_TAGS='psaspect', STRUC_ADD={psaspect:FLOAT(pplot.psaspect)}
COPY_STRUC, pplot, pplot, EXCEPT_TAGS='dummy'
IF nplots EQ 0 THEN BEGIN ; Preparacin de ejes para el caso (SINGLEPLOT) en que slo hay fondo (se generan range y style)
 nplots = 1
 IF KEYWORD_SET(BACKGROUND) EQ 0 THEN BEGIN
  MESSAGE, 'At least one set of data is needed'
 ENDIF ELSE BEGIN
  IF TAG_EXIST(xxl,'range') EQ 0 THEN $
   IF KEYWORD_SET(XRANGE)   EQ 0 THEN xxl = CREATE_STRUCT(xxl, {range:[      0.0,    npixx]}) $
                                 ELSE xxl = CREATE_STRUCT(xxl, {range:xrange               })
  IF TAG_EXIST(yyl,'range') EQ 0 THEN $
   IF KEYWORD_SET(YRANGE)   EQ 0 THEN yyl = CREATE_STRUCT(yyl, {range:[      0.0,    npixy]}) $
                                 ELSE yyl = CREATE_STRUCT(yyl, {range:yrange               })
  IF N_ELEMENTS(xxl.range)  NE 2 THEN MESSAGE, 'Incorrect dimensions for xaxisl.range'
  IF N_ELEMENTS(yyl.range)  NE 2 THEN MESSAGE, 'Incorrect dimensions for yaxisl.range'
  IF SIZE(xxl.range,/TYPE)  NE 4 AND $
     SIZE(xxl.range,/TYPE)  NE 5 THEN COPY_STRUC, xxl, xxl, EXCEPT_TAGS='range', STRUC_ADD={range:FLOAT(xxl.range)}
  IF SIZE(yyl.range,/TYPE)  NE 4 AND $
     SIZE(yyl.range,/TYPE)  NE 5 THEN COPY_STRUC, yyl, yyl, EXCEPT_TAGS='range', STRUC_ADD={range:FLOAT(yyl.range)}
  v1 = {x:[3.*xxl.range[1]+2.*xxl.range[0]], y:[3.*yyl.range[1]+2.*yyl.range[0]]}
  IF TAG_EXIST(xxl,'style') EQ 0 THEN $
   IF KEYWORD_SET(DAXIS)    EQ 0 THEN xxl = CREATE_STRUCT(xxl, {style:     -2 }) $
                                 ELSE xxl = CREATE_STRUCT(xxl, {style:daxis[1]})
  IF TAG_EXIST(yyl,'style') EQ 0 THEN $
   IF KEYWORD_SET(DAXIS)    EQ 0 THEN yyl = CREATE_STRUCT(yyl, {style:     -2 }) $
                                 ELSE yyl = CREATE_STRUCT(yyl, {style:daxis[0]})
  IF TAG_EXIST(xxu,'range') EQ 0 THEN xxu = CREATE_STRUCT(xxu, {range:xxl.range})
  IF TAG_EXIST(yyu,'range') EQ 0 THEN yyu = CREATE_STRUCT(yyu, {range:yyl.range})
  IF N_ELEMENTS(xxu.range)  NE 2 THEN MESSAGE, 'Incorrect dimensions for xaxisu.range'
  IF N_ELEMENTS(yyu.range)  NE 2 THEN MESSAGE, 'Incorrect dimensions for yaxisu.range'
  IF SIZE(xxu.range,/TYPE)  NE 4 AND $
     SIZE(xxu.range,/TYPE)  NE 5 THEN COPY_STRUC, xxu, xxu, EXCEPT_TAGS='range', STRUC_ADD={range:FLOAT(xxu.range)}
  IF SIZE(yyu.range,/TYPE)  NE 4 AND $
     SIZE(yyu.range,/TYPE)  NE 5 THEN COPY_STRUC, yyu, yyu, EXCEPT_TAGS='range', STRUC_ADD={range:FLOAT(yyu.range)}
  IF TAG_EXIST(xxu,'style') EQ 0 THEN $
   IF KEYWORD_SET(DAXIS)    EQ 0 THEN xxu = CREATE_STRUCT(xxu, {style:     -2 }) $
                                 ELSE xxu = CREATE_STRUCT(xxu, {style:daxis[3]})
  IF TAG_EXIST(yyu,'style') EQ 0 THEN $
   IF KEYWORD_SET(DAXIS)    EQ 0 THEN yyu = CREATE_STRUCT(yyu, {style:     -2 }) $
                                 ELSE yyu = CREATE_STRUCT(yyu, {style:daxis[2]})
 ENDELSE
ENDIF ELSE BEGIN ; Preparacin de ejes para caso en que no hay fondo (se genera style)
 IF TAG_EXIST(xxl,'style')  EQ 0 THEN $
  IF KEYWORD_SET(DAXIS)     EQ 0 THEN xxl = CREATE_STRUCT(xxl, {style:      1 }) $
                                 ELSE xxl = CREATE_STRUCT(xxl, {style:daxis[1]})
 IF TAG_EXIST(yyl,'style')  EQ 0 THEN $
  IF KEYWORD_SET(DAXIS)     EQ 0 THEN yyl = CREATE_STRUCT(yyl, {style:      1 }) $
                                 ELSE yyl = CREATE_STRUCT(yyl, {style:daxis[0]})
 IF TAG_EXIST(xxu,'style')  EQ 0 THEN $
  IF KEYWORD_SET(DAXIS)     EQ 0 THEN xxu = CREATE_STRUCT(xxu, {style:      0 }) $
                                 ELSE xxu = CREATE_STRUCT(xxu, {style:daxis[3]})
 IF TAG_EXIST(yyu,'style')  EQ 0 THEN $
  IF KEYWORD_SET(DAXIS)     EQ 0 THEN yyu = CREATE_STRUCT(yyu, {style:      0 }) $
                                 ELSE yyu = CREATE_STRUCT(yyu, {style:daxis[2]})
ENDELSE
IF N_ELEMENTS(xxl.style)    NE 1 THEN MESSAGE, 'Incorrect dimensions for xaxisl.style'
IF N_ELEMENTS(yyl.style)    NE 1 THEN MESSAGE, 'Incorrect dimensions for yaxisl.style'
IF N_ELEMENTS(xxu.style)    NE 1 THEN MESSAGE, 'Incorrect dimensions for xaxisu.style'
IF N_ELEMENTS(yyu.style)    NE 1 THEN MESSAGE, 'Incorrect dimensions for yaxisu.style'
COPY_STRUC, xxl, xxl, EXCEPT_TAGS='dummy'
COPY_STRUC, yyl, yyl, EXCEPT_TAGS='dummy'
COPY_STRUC, xxu, xxu, EXCEPT_TAGS='dummy'
COPY_STRUC, yyu, yyu, EXCEPT_TAGS='dummy'
END
