;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; COPY_STRUC v. 3.1, 11 May 2013.                                              ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure copies one structure to a new one possibly excluding some     ;
;  tags and adding the tags in another structure.                              ;
;                                                                              ;
; Positional parameters:                                                       ;
; struc_from:    Input structure.                                              ;
; struc_to:      Output structure.                                             ;
;                                                                              ;
; Keyword parameters:                                                          ;
; EXCEPT_TAGS:   Tags to be excluded.                                          ;
; STRUC_ADD:     Structure with tags to be copied.                             ;
; VERBOSE:       Verbose flag.                                                 ;
;                                                                              ;
; Version history:                                                             ;
; v3.0           First version.                                                ;
; v3.1:          Only version change.                                          ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO COPY_STRUC, struc_from, struc_to, $
    EXCEPT_TAGS=except_tags, STRUC_ADD=struc_add, VERBOSE=verbose
ON_ERROR, 1
n_from      = N_TAGS(struc_from)
n_exc       = N_ELEMENTS(except_tags)
IF n_from LT 1 THEN MESSAGE, 'You need at least one tag in struc_from'
IF n_exc EQ 0 THEN BEGIN
 struc_tmp  = struc_from
ENDIF ELSE BEGIN
 tags       = STRLOWCASE(TAG_NAMES(struc_from))
 str        =                        "tags EQ STRLOWCASE('" + except_tags[0] + "')"
 FOR i=1,n_exc -1 DO str = str + " OR tags EQ STRLOWCASE('" + except_tags[i] + "')"
 str        = 'posc = WHERE(' + str + ', npos, COMP=pos2, NCOMP=npos2)'
 tstr       = EXECUTE(str)
 IF npos2 NE 0 THEN BEGIN
  tstr      = EXECUTE('struc_tmp = CREATE_STRUCT({'           + tags[pos2[0]] + ':struc_from.(pos2[0])})')
  IF KEYWORD_SET(VERBOSE) THEN PRINT, 'Starting with TAG: '   + tags[pos2[0]]
  FOR i=1,npos2-1 DO BEGIN
   tstr     = EXECUTE('struc_tmp = CREATE_STRUCT(struc_tmp,{' + tags[pos2[i]] + ':struc_from.(pos2[i])})')
   IF KEYWORD_SET(VERBOSE) THEN PRINT, 'Adding TAG: '         + tags[pos2[i]]
  ENDFOR
 ENDIF ELSE BEGIN
  struc_tmp = {dummy:0}
 ENDELSE
ENDELSE
IF N_ELEMENTS(STRUC_ADD) NE 0 THEN BEGIN
 tags2      = STRLOWCASE(TAG_NAMES(struc_add))
 n_add      = N_ELEMENTS(tags2)
 FOR i=0,n_add-1 DO BEGIN
  IF TAG_EXIST(struc_tmp,'dummy') EQ 0 OR tags2[i] NE 'dummy' THEN $ ; Avoiding "dummy conflicts"
   tstr     = EXECUTE('struc_tmp = CREATE_STRUCT(struc_tmp,{' +     tags2[i]  + ':struc_add.(i)})')
  IF KEYWORD_SET(VERBOSE) THEN PRINT, 'Adding TAG: '         +     tags2[i]
 ENDFOR
ENDIF
struc_to    = struc_tmp
END
