;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; DOAXIS v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure processes the axes and adds the following (internal) tags to  ;
;  the estructures:                                                            ;
;                           type                                               ;
;                           delta                                              ;
;                           range2                                             ;
;                           length                                             ;
;                           minorv                                             ;
;                           tickname2                                          ;
;                           log2                                               ;
;                                                                              ;
; Positional parameters:                                                       ;
; v:         Axis structure.                                                   ;
; type:      'xl', 'yl', 'xu', or 'yu'.                                        ;
;                                                                              ;
; Keyword parameters:                                                          ;
; VL:        Lower axis structure (only applies when processing an upper axis).;
; LOG:       Logarithmic axis flag.                                            ;
; EXTRA:     If 1.0, it increases the x range by 10%. If not 1.0, it increases ;
;             the natural x range by that fraction.                            ;
; MINMAX:    [xmin,ymin,xmax,ymax] values for the x and y ranges.              ;
; TICKF:     Scale factor for the tick lengths.                                ;
; TITLE      Axis title.                                                       ;
; LESSTICKS: Flag to use less ticks.                                           ;
; MORETICKS: Flag to use more ticks.                                           ;
; EXTTICKS:  String array to fix the text for the ticks externally.            ;
; SPANISH:   Flag to use decimal commas instead of decimal points.             ;
;                                                                              ;
; Version history:                                                             ;
; v3.0       First version.                                                    ;
; v3.1:      Only version change.                                              ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO DOAXIS, v, type, VL=vl, LOG=log, EXTRA=extra, MINMAX=minmax, TICKF=tickf,  $
	             TITLE=title, MORETICKS=moreticks, LESSTICKS=lessticks,    $
	             EXTTICKS=extticks, SPANISH=spanish
COMMON vv,          vv1,  vv2,  vv3,  vv4,  vv5,  vv6,  vv7,  vv8,  vv9, vv10, $
                   vv11, vv12, vv13, vv14, vv15, vv16, vv17, vv18, vv19, vv20, $
                   vv21, vv22, vv23, vv24, vv25, vv26, vv27, vv28, vv29, vv30, $
                   vv31, vv32, vv33, vv34, vv35, vv36, vv37, vv38, vv39, vv40, $
                   vv41, vv42, vv43, vv44, vv45, vv46, vv47, vv48, vv49, vv50, $
                   vv51, vv52, vv53, vv54, vv55, vv56, vv57, vv58, vv59, vv60
ON_ERROR, 1
;
; Procesamos el valor del eje
;
typexy = STRMID(type,0,1)
typelu = STRMID(type,1,1)
ixy    = 1*(typexy EQ 'y')
IF N_ELEMENTS(vl) EQ 0 THEN vl = {dummy:0}
;
; Etiqueta de tipo de eje
;
IF TAG_EXIST(v,'type')    EQ 0 THEN v = CREATE_STRUCT(v, { type:type    })
IF N_ELEMENTS(v.type)     NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.type' 
;
; Etiquetas de logaritmo y de rango determinado a partir de datos + expansin
;
IF TAG_EXIST(v,'log'  )   EQ 0 THEN $                                             ; 1a opcin: etiqueta
 IF TAG_EXIST(vl,'log')        THEN v = CREATE_STRUCT(v, {  log:vl.log  }) ELSE $ ; 2a opcin: eje inferior
 IF KEYWORD_SET(LOG)           THEN v = CREATE_STRUCT(v, {  log:log     }) ELSE $ ; 3a opcin: bandera
                                    v = CREATE_STRUCT(v, {  log:       0})        ; 4a opcin: valor por defecto
IF N_ELEMENTS(v.log)      NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.log'   
IF KEYWORD_SET(EXTRA)     EQ 0 THEN extra = 0.0 ELSE IF extra EQ 1.0 THEN extra = -1.0
IF TAG_EXIST(v,'extra')   EQ 0 THEN $
 IF TAG_EXIST(vl,'extra')      THEN v = CREATE_STRUCT(v, {extra:vl.extra}) ELSE $
 IF KEYWORD_SET(EXTRA)         THEN v = CREATE_STRUCT(v, {extra:extra   }) ELSE $
                                    v = CREATE_STRUCT(v, {extra:     0.0})
IF N_ELEMENTS(v.extra)    NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.extra' 
IF SIZE(v.extra,/TYPE)    NE 4 AND $
   SIZE(v.extra,/TYPE)    NE 5 THEN COPY_STRUC, v, v, EXCEPT_TAGS='extra', STRUC_ADD={extra:FLOAT(v.extra)}
v.extra = v.extra*(v.extra GT -1.0) + 0.1*(v.extra LE -1.0)
;
; Calculamos el mnimo y el mximo del eje
;
IF TAG_EXIST(v,'range') OR TAG_EXIST(vl,'range') OR KEYWORD_SET(MINMAX) THEN BEGIN ; 1a-3a opcin
 IF TAG_EXIST(v,'range')  EQ 1 AND typelu EQ 'u' THEN BEGIN
  IF v.range[0] NE vl.range[0] OR v.range[1] NE vl.range[1] THEN $
   MESSAGE, 'Range cannot be specified for upper axis: use tickv and tickname instead'
 ENDIF
 IF TAG_EXIST(v,'range')  EQ 0 THEN $ 
  IF TAG_EXIST(vl,'range')     THEN v = CREATE_STRUCT(v, {range:vl.range                     }) $
                               ELSE v = CREATE_STRUCT(v, {range:[minmax[0+ixy],minmax[2+ixy]]})
 IF N_ELEMENTS(v.range)   NE 2 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.range' 
 IF SIZE(v.range,/TYPE)   NE 4 AND $
    SIZE(v.range,/TYPE)   NE 5 THEN COPY_STRUC, v, v, EXCEPT_TAGS='range', STRUC_ADD={range:FLOAT(v.range)}
 delta = ABS(v.range[1] - v.range[0])
ENDIF ELSE BEGIN                                                                   ; 4a opcin
 tmp   = EXECUTE('vm = vv1.' + typexy + ' - vv1.s' + typexy + 'm + vv1.d' + typexy + '*(vv1.d' + typexy + ' LE 0.0)')
 tmp   = EXECUTE('vp = vv1.' + typexy + ' + vv1.s' + typexy + 'p + vv1.d' + typexy + '*(vv1.d' + typexy + ' GE 0.0)')
 FOR i=1,vv1.nplots-1 DO BEGIN
  is   = STRTRIM(i+1,2)
  tmp  = EXECUTE('vm = [vm, vv' + is + '.' + typexy + ' - vv' + is + '.s' + typexy + 'm + ' + $ 
	                   'vv' + is + '.d' + typexy + '*(vv' + is + '.d' + typexy + ' LE 0.0)]')
  tmp  = EXECUTE('vp = [vp, vv' + is + '.' + typexy + ' + vv' + is + '.s' + typexy + 'p + ' + $ 
	                   'vv' + is + '.d' + typexy + '*(vv' + is + '.d' + typexy + ' GE 0.0)]')
 ENDFOR
 MINMAX2, vm, vp, vmin, vmax, delta, EXTRAV=v.extra, LOG=v.log
 v     = CREATE_STRUCT(v, {range:[vmin,vmax]})
ENDELSE
IF v.range[0] EQ v.range[1] THEN BEGIN ; Caso tonto
 delta      = 1.e-6
 v.range[1] = v.range[0]
ENDIF
IF TAG_EXIST(v,'delta')  EQ 0 THEN v = CREATE_STRUCT(v, { delta:delta                      })
IF TAG_EXIST(v,'range2') EQ 0 THEN v = CREATE_STRUCT(v, {range2:[MIN(v.range),MAX(v.range)]})
IF type EQ 'xl' THEN !X.RANGE = v.range ELSE $
IF type EQ 'yl' THEN !Y.RANGE = v.range
;
; Etiquetas de longitud de marcas, grosor, estilo de lnea, ttulo y tamao de caracteres
;
IF TAG_EXIST(v,'ticklen')    EQ 0 THEN $
 IF TAG_EXIST(vl,'ticklen')       THEN v = CREATE_STRUCT(v, {  ticklen:vl.ticklen  }) ELSE $
 IF KEYWORD_SET(TICKF)            THEN v = CREATE_STRUCT(v, {  ticklen:tickf       }) ELSE $
                                       v = CREATE_STRUCT(v, {  ticklen:         1.0})
IF N_ELEMENTS(v.ticklen)     NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.ticklen' 
IF SIZE(v.ticklen,/TYPE)     NE 4 AND $
   SIZE(v.ticklen,/TYPE)     NE 5 THEN COPY_STRUC, v, v, EXCEPT_TAGS='ticklen',  STRUC_ADD={ ticklen:FLOAT(v.ticklen) }
IF TAG_EXIST(v,'thick')      EQ 0 THEN $
 IF TAG_EXIST(vl,'thick')         THEN v = CREATE_STRUCT(v, {    thick:vl.thick    }) ELSE $
                                       v = CREATE_STRUCT(v, {    thick:         1.0})
IF N_ELEMENTS(v.thick)       NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.thick'
IF SIZE(v.thick,/TYPE)       NE 4 AND $
   SIZE(v.thick,/TYPE)       NE 5 THEN COPY_STRUC, v, v, EXCEPT_TAGS='thick'  ,  STRUC_ADD={   thick:FLOAT(v.thick  ) }
IF TAG_EXIST(v,'linestyle')  EQ 0 THEN $
 IF TAG_EXIST(vl,'linestyle')     THEN v = CREATE_STRUCT(v, {linestyle:vl.linestyle}) ELSE $
                                       v = CREATE_STRUCT(v, {linestyle:           0})
IF N_ELEMENTS(v.thick)       NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.linestyle'


IF TAG_EXIST(v,'title')      EQ 0 THEN $
 IF TAG_EXIST(vl,'title')         THEN v = CREATE_STRUCT(v, {    title:vl.title    }) ELSE $
 IF KEYWORD_SET(TITLE)            THEN v = CREATE_STRUCT(v, {    title:title       }) ELSE $
                                       v = CREATE_STRUCT(v, {    title:          ''})
IF N_ELEMENTS(v.title)       NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.title'   
IF TAG_EXIST(v,'charsize')   EQ 0 THEN $
 IF TAG_EXIST(vl,'charsize')      THEN v = CREATE_STRUCT(v, { charsize:vl.charsize }) ELSE $
                                       v = CREATE_STRUCT(v, { charsize:         1.0})
IF N_ELEMENTS(v.charsize)    NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.charsize'
IF SIZE(v.charsize,/TYPE)    NE 4 AND $
   SIZE(v.charsize,/TYPE)    NE 5 THEN COPY_STRUC, v, v, EXCEPT_TAGS='charsize', STRUC_ADD={charsize:FLOAT(v.ticklen )}
;
; Etiqueta de malla
;
IF TAG_EXIST(v,'grid')       EQ 0 THEN v = CREATE_STRUCT(v, {     grid:           0})
IF N_ELEMENTS(v.grid)        NE 1 THEN MESSAGE, 'Incorrect dimensions for ' + type + '.grid' 
;
; Texto externo de las marcas (no se crea si no existe)
;
IF TAG_EXIST(v,'tickname')  EQ 0 THEN $
 IF TAG_EXIST(vl,'tickname')     THEN v = CREATE_STRUCT(v, {  tickname:vl.tickname }) ELSE $
 IF KEYWORD_SET(EXTTICKS)        THEN v = CREATE_STRUCT(v, {  tickname:extticks    })
;
; Nmero y valores de las marcas
;
IF TAG_EXIST(vl,'range') EQ 0 THEN vl = CREATE_STRUCT(vl, {range:RANDOMN(seed,2)}) ; Dummy
IF TAG_EXIST(vl,'log'  ) EQ 0 THEN vl = CREATE_STRUCT(vl, {  log:-99            }) ; Dummy
; Caso [1]: Se copian los valores del eje inferior si no hay cambios
IF TAG_EXIST(v,'tickv')  + TAG_EXIST(v,'lessticks') + TAG_EXIST(v,'moreticks')       + $
   TAG_EXIST(v,'minorn') + ABS(v.range[1]-vl.range[1]) + ABS(v.range[0]-vl.range[0]) + $
                                    ABS(ROUND(0.5*v.log+0.1)-ROUND(0.5*vl.log+0.1)) EQ 0 THEN BEGIN
 IF TAG_EXIST(v,'tickname') THEN tickname2 = v.tickname ELSE tickname2 = vl.tickname2
 v = CREATE_STRUCT(v, {tickv:vl.tickv, tickname2:tickname2, lessticks:vl.lessticks, moreticks:vl.moreticks, $
	               minorn:vl.minorn, minorv:vl.minorv, length:LENGTHV(tickname2)})
; Caso [2]: Se dan los valores manualmente
ENDIF ELSE IF TAG_EXIST(v,'tickv') THEN BEGIN
 pos     = WHERE(v.tickv GE v.range2[0] AND v.tickv LE v.range2[1],npos)
 IF npos EQ 0 THEN MESSAGE, 'All your specified tick values are outside the allowed range'
 COPY_STRUC, v, v, EXCEPT_TAGS=['tickv'], STRUC_ADD={tickv:v.tickv[pos]}
 IF TAG_EXIST(v,'tickname') THEN tickname2 = v.tickname $
	                    ELSE tickname2 = FTICKS(v.tickv, LOG=v.log, SPANISH=spanish)
 IF TAG_EXIST(v,'minorn') EQ 0 THEN v = CREATE_STRUCT(v, {minorn:1})
 IF v.minorn LE 0 THEN v.minorn = 1
 tickvn  = N_ELEMENTS(v.tickv)
 IF TAG_EXIST(v,'minorv') EQ 0 THEN BEGIN
  IF tickvn EQ 1 OR v.minorn EQ 1 THEN BEGIN
   minorv = v.tickv
  ENDIF ELSE BEGIN
   tickvs = v.tickv[SORT(v.tickv)]
   minorv = tickvs[0] - 1000*(tickvs[1]-tickvs[0])        + (tickvs[       1]-tickvs[       0])* FINDGEN(1000*v.minorn+1 )/v.minorn
   FOR i=0,tickvn-2 DO minorv = [minorv, tickvs[i]        + (tickvs[     i+1]-tickvs[       i])*(FINDGEN(     v.minorn)+1)/v.minorn]
   minorv =                     [minorv, tickvs[tickvn-1] + (tickvs[tickvn-1]-tickvs[tickvn-2])*(FINDGEN(1000*v.minorn)+1)/v.minorn]
   pos    = WHERE(minorv GE v.range2[0] AND minorv LE v.range2[1])
   minorv = minorv[pos]
  ENDELSE
 ENDIF ELSE BEGIN
  minorv  = v.minorv
 ENDELSE
 COPY_STRUC, v, v, EXCEPT_TAGS=['tickname2','lessticks','moreticks','minorv','length'], $
	             STRUC_ADD={tickname2:tickname2,lessticks:0,moreticks:0, $
			        minorv:minorv,length:LENGTHV(tickname2)}
; Caso [3]: Clculo y formateado normal
ENDIF ELSE BEGIN
 IF (TAG_EXIST(v,'lessticks')+KEYWORD_SET(LESSTICKS))* $
    (TAG_EXIST(v,'moreticks')+KEYWORD_SET(MORETICKS)) NE 0 THEN MESSAGE, 'Incorrect tick tags/flags in ' + type
 IF typexy EQ 'x' THEN BEGIN
  IF TAG_EXIST(v,'lessticks') OR KEYWORD_SET(LESSTICKS) THEN BEGIN
   v1 = 2.0   & v2 = 2.0   & v3 = 5.0   & v4 = 1.0e2
   COPY_STRUC, v, v, EXCEPT_TAGS='lessticks', STRUC_ADD={lessticks:1}
  ENDIF ELSE BEGIN
   v1 = 2.0   & v2 = 5.0   & v3 = 1.0e2 & v4 = 1.0e4
   COPY_STRUC, v, v, EXCEPT_TAGS='lessticks', STRUC_ADD={lessticks:0}
  ENDELSE
 ENDIF ELSE BEGIN
  IF TAG_EXIST(v,'lessticks') OR KEYWORD_SET(LESSTICKS) THEN BEGIN
   v1 = 2.0   & v2 = 2.0   & v3 = 20.0  & v4 = 1.0e4
   COPY_STRUC, v, v, EXCEPT_TAGS='lessticks', STRUC_ADD={lessticks:1}
  ENDIF ELSE BEGIN
   v1 = 2.0   & v2 = 5.0   & v3 = 1.0e4 & v4 = 1.0e7
   COPY_STRUC, v, v, EXCEPT_TAGS='lessticks', STRUC_ADD={lessticks:0}
  ENDELSE
 ENDELSE
 IF TAG_EXIST(v,'moreticks') OR KEYWORD_SET(MORETICKS) THEN $
  COPY_STRUC, v, v, EXCEPT_TAGS='moreticks', STRUC_ADD={moreticks:1} ELSE $
  COPY_STRUC, v, v, EXCEPT_TAGS='moreticks', STRUC_ADD={moreticks:0}
 IF TAG_EXIST(v,'minorn') EQ 0 THEN v = CREATE_STRUCT(v, {minorn:5})
 IF v.minorn LE 0 THEN v.minorn = 1
 TICKS2, v, v1, v2, v3, v4, SPANISH=spanish
ENDELSE
IF N_ELEMENTS(v.tickname2) NE N_ELEMENTS(v.tickv) THEN $
 MESSAGE, 'Number of text tick marks does not correspond to number of tick values'
;COPY_STRUC, v, v, EXCEPT_TAGS=['tickname','log2'], $
;	          STRUC_ADD={tickname:v.tickname2, log2:v.log*(v.log LE 1) + 1*(v.log GT 1)}
COPY_STRUC, v, v, EXCEPT_TAGS='log2', STRUC_ADD={log2:v.log*(v.log LE 1) + 1*(v.log GT 1)}
v.length = v.length*(v.style EQ 1)
;
; Anulamos los ttulos por defecto
;
!X.TITLE = ''
!Y.TITLE = ''
END

