;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; FTICKS v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This function formats and returns the text for the tick marks.               ;
;                                                                              ;
; Positional parameters:                                                       ;
; value:   Array with the values for the text.                                 ;
;                                                                              ;
; Keyword parameters:                                                          ;
; LOG:     Flag for logarithmic axes.                                          ;
; SPANISH: Flag to use decimal commas instead of decimal points.               ;
; NOTRIM:  Flag to not eliminate the spaces at the beginning of the strings.   ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:   Roundoff errors corrected.                                           ;
;         NOTRIM flag added.                                                   ;
; v0.2:   More roundoff errors corrected.                                      ;
;         LOG=2 added (see SINGLEPLOT).                                        ;
; v1.0:   General clean up.                                                    ;
; v1.1:   Bug with multypling dot for PS fonts fixed.                          ;
; v2.0:   English version.                                                     ;
; v2.1:   IDLdoc documentation.                                                ;
; v2.5:   Version system update.                                               ;
; v3.0:   Adapted to global changes in SINGLEPLOT and SINGLECONTOUR.           ;
; v3.1:   Only version change.                                                 ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
FUNCTION FTICKS, value, LOG=log, SPANISH=spanish, NOTRIM=notrim, FONT=font
ON_ERROR, 1
;
; Inicializacin de datos y parmetros
;
IF KEYWORD_SET(SPANISH) EQ 0 THEN spanish = 0
IF KEYWORD_SET(FONT)    EQ 0 THEN font    = 0
IF KEYWORD_SET(FONT)    EQ 0 THEN mul     = '' ELSE mul = ''
IF KEYWORD_SET(NOTRIM)  EQ 0 THEN notrim  = 0
IF KEYWORD_SET(LOG)     EQ 0 THEN log     = 0
v       = DOUBLE(value)
n       = N_ELEMENTS(v)
;
; Hallamos los mnimos y mximos para ver si se utiliza el formato cientfico
; o no. Tambin creamos un vector vv idntico a v substituyendo el valor 0
; (si existe) por 1.0 para hacer comparaciones para hallar el nmero de 
; decimales que se han de utilizar sin caer en problemas de redondeo.
;
vmin    = MIN(v,MAX=vmax)
vabsmin = MIN(ABS(v),posmin)
vv      = v
IF vabsmin EQ 0.0 THEN FOR i=0,n-1 DO IF vv[i] EQ 0.0 THEN vv[i]=1.0
vc = 1.0e-6
;
; Aqu se decide si se usa el formato cientfico o no. Ntese que para ejes
; logartmicos se usa siempre si log = 1 (pero puede ser 2, en cuyo caso no
; ocurre eso).
;
     IF (log EQ 1 OR vmin LE -1.0e5 OR vmax GE 1.0e5 OR $
                            (vmin GT -1.0e-3 AND vmax LT 1.0e-3)) THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Empieza formato cientfico          ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Caso de 5 o ms decimales (a partir del sexto no se muestra)
;
 vs4   = STRING(v, FORMAT='(e11.4)') 
 dmax  = MAX(ABS(v-vs4)/ABS(vv))
 IF dmax GT vc THEN BEGIN
  vs5   = STRING(v, FORMAT='(e12.5)') 
  RETURN, FTICKSAUX(vs5, 5, SPANISH=spanish, FONT=font)
 ENDIF 
;
; Caso de 4 decimales
;
 vs3   = STRING(v, FORMAT='(e10.3)') 
 dmax  = MAX(ABS(v-vs3)/ABS(vv))
 IF dmax GT vc THEN RETURN, FTICKSAUX(vs4, 4, SPANISH=spanish, FONT=font)
;
; Caso de 3 decimales
;
 vs2   = STRING(v, FORMAT='(e9.2)') 
 dmax  = MAX(ABS(v-vs2)/ABS(vv))
 IF dmax GT vc THEN RETURN, FTICKSAUX(vs3, 3, SPANISH=spanish, FONT=font)
;
; Caso de 2 decimales
;
 vs1   = STRING(v, FORMAT='(e8.1)') 
 dmax  = MAX(ABS(v-vs1)/ABS(vv))
 IF dmax GT vc THEN RETURN, FTICKSAUX(vs2, 2, SPANISH=spanish, FONT=font)
;
; Caso de 1 decimal
;
 vs0   = STRING(v, FORMAT='(e7.0)') 
 dmax  = MAX(ABS(v-vs0)/ABS(vv))
 IF dmax GT vc THEN RETURN, FTICKSAUX(vs1, 1, SPANISH=spanish, FONT=font)
;
; Caso de ningn decimal pero no todo potencias de diez o 
;                        con algn valor negativo
;
 vv    = STRARR(n)
 dmax  = MAX(ABS(STRMID(vs0,1,1)-1.0))
 IF dmax GT vc OR vmin LT 0.0 THEN BEGIN
  FOR i=0,n-1 DO BEGIN
   IF STRMID(vs0(i),5,1) EQ '0' THEN BEGIN
;
; Subcasos de exponente positivo/negativo de un dgito
;
    IF STRMID(vs0(i),4,1) EQ '+' THEN $
     vv(i)=STRMID(vs0(i),0,2)+mul+'!X10!U'+STRMID(vs0(i),6,1)+'!N' ELSE $
     vv(i)=STRMID(vs0(i),0,2)+mul+'!X10!U-'+STRMID(vs0(i),6,1)+'!N'
   ENDIF ELSE BEGIN
;
; Subcasos de exponente positivo/negativo de dos dgitos
;
    IF STRMID(vs0(i),4,1) EQ '+' THEN $
     vv(i)=STRMID(vs0(i),0,2)+mul+'!X10!U'+STRMID(vs0(i),5,2)+'!N' ELSE $
     vv(i)=STRMID(vs0(i),0,2)+mul+'!X10!U'+STRMID(vs0(i),4,3)+'!N'
   ENDELSE
   IF FLOAT(vs0(i)) EQ 0.0 THEN vv(i)='0'
  ENDFOR
;
; El primer carcter debe ser un espacio o un signo -. El espacio se elimina
; con STRTRIM.
;
  IF KEYWORD_SET(NOTRIM) THEN RETURN, vv ELSE RETURN, STRTRIM(vv,1)
 ENDIF 
;
; Caso de todo potencias de diez positivas
;
 vs00  = STRING(v, FORMAT='(e6.0)') 
 FOR i=0,n-1 DO BEGIN
  IF STRMID(vs00(i),4,1) EQ '0' THEN BEGIN
;
; Subcasos de exponente positivo/negativo de un dgito
;
   IF STRMID(vs00(i),3,1) EQ '+' THEN $
    vv(i)='10!U'+STRMID(vs00(i),5,1)+'!N' ELSE $
    vv(i)='10!U-'+STRMID(vs00(i),5,1)+'!N'
  ENDIF ELSE BEGIN
;
; Subcasos de exponente positivo/negativo de dos dgitos
;
   IF STRMID(vs00(i),3,1) EQ '+' THEN $
    vv(i)='10!U'+STRMID(vs00(i),4,2)+'!N' ELSE $
    vv(i)='10!U'+STRMID(vs00(i),3,3)+'!N'
  ENDELSE
 ENDFOR
;
; En este caso no hay puntos decimales por los que preocuparse para el formato
; espaol.
;
 RETURN, vv
ENDIF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Empieza formato no cientfico       ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Caso de 5 o ms decimales (a partir del sexto no se muestra)
;
vs4   = STRING(v, FORMAT='(f11.4)') 
dmax  = MAX(ABS(v-vs4)/ABS(vv)) 
IF dmax GT vc THEN BEGIN
 vs5   = STRING(v, FORMAT='(f12.5)')
 IF KEYWORD_SET(SPANISH) THEN vs5 = CHPOINTCOMMA(vs5)
 IF KEYWORD_SET(NOTRIM) THEN RETURN, vs5 ELSE RETURN, STRTRIM(vs5,1)
ENDIF
;
; Caso de 4 decimales
;
vs3   = STRING(v, FORMAT='(f10.3)')
dmax  = MAX(ABS(v-vs3)/ABS(vv))
IF KEYWORD_SET(SPANISH) THEN vs4 = CHPOINTCOMMA(vs4)
IF dmax GT vc AND KEYWORD_SET(NOTRIM)      THEN RETURN, vs4
IF dmax GT vc AND KEYWORD_SET(NOTRIM) EQ 0 THEN RETURN, STRTRIM(vs4,1)
;
; Caso de 3 decimales
;
vs2   = STRING(v, FORMAT='(f9.2)')
dmax  = MAX(ABS(v-vs2)/ABS(vv))
IF KEYWORD_SET(SPANISH) THEN vs3 = CHPOINTCOMMA(vs3)
IF dmax GT vc AND KEYWORD_SET(NOTRIM)      THEN RETURN, vs3
IF dmax GT vc AND KEYWORD_SET(NOTRIM) EQ 0 THEN RETURN, STRTRIM(vs3,1)
;
; Caso de 2 decimales
;
vs1   = STRING(v, FORMAT='(f8.1)')
dmax  = MAX(ABS(v-vs1)/ABS(vv))
IF KEYWORD_SET(SPANISH) THEN vs2 = CHPOINTCOMMA(vs2)
IF dmax GT vc AND KEYWORD_SET(NOTRIM)      THEN RETURN, vs2
IF dmax GT vc AND KEYWORD_SET(NOTRIM) EQ 0 THEN RETURN, STRTRIM(vs2,1)
;
; Caso de 1 decimal
;
vs0   = STRING(v, FORMAT='(f7.0)')
dmax  = MAX(ABS(v-vs0)/ABS(vv))
IF KEYWORD_SET(SPANISH) THEN vs1 = CHPOINTCOMMA(vs1)
IF dmax GT vc AND KEYWORD_SET(NOTRIM)      THEN RETURN, vs1
IF dmax GT vc AND KEYWORD_SET(NOTRIM) EQ 0 THEN RETURN, STRTRIM(vs1,1)
;
; Caso de nicamente enteros (aqu no hace falta cambiar puntos decimales por
; comas)
;
IF KEYWORD_SET(NOTRIM) THEN RETURN, STRMID(vs0,0,6)            ELSE $
                            RETURN, STRTRIM(STRMID(vs0,0,6),1)

END
