;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; FTICKSAUX v. 3.1, 11 May 2013.                                               ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; Auxiliary function for FTICKS.                                               ;
;                                                                              ;
; Positional parameters:                                                       ;
; vs:      String array with the text for the ticks.                           ;
; nd:      Number of digits after the decimal marker in vs.                    ;
;                                                                              ;
; Keyword parameters:                                                          ;
; SPANISH: Flag to use decimal commas instead of decimal points.               ;
; NOTRIM:  Flag to not eliminate the spaces at the beginning of the strings.   ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:   NOTRIM flag added.                                                   ;
; v1.0:   General clean up.                                                    ;
; v1.1:   Bug with multypling dot for PS fonts fixed.                          ;
; v2.0:   English version.                                                     ;
; v2.1:   IDLdoc documentation.                                                ;
; v2.5:   Version system update.                                               ;
; v3.0:   Adapted to global changes in SINGLEPLOT and SINGLECONTOUR.           ;
; v3.1:   Only version change.                                                 ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
FUNCTION FTICKSAUX, vs, nd, SPANISH=spanish, NOTRIM=notrim, FONT=font
ON_ERROR, 1
;
; Inicializamos variable y banderas.
; 
IF KEYWORD_SET(SPANISH) EQ 0 THEN spanish=0
IF KEYWORD_SET(FONT) EQ 0 THEN mul = '' ELSE mul = ''
n     = N_ELEMENTS(vs)
vv    = STRARR(n)
;
; Empieza el bucle principal.       
; 
FOR i=0,n-1 DO BEGIN
 IF STRMID(vs(i),nd+5,1) EQ '0' THEN BEGIN
  IF STRMID(vs(i),nd+4,1) EQ '+' THEN $
;
; Caso de exponente positivo de un dgito
;
   vv(i)=STRMID(vs(i),0,nd+3)+mul+'!X10!U'+STRMID(vs(i),nd+6,1)+'!N' ELSE $
;
; Caso de exponente negativo de un dgito
;
   vv(i)=STRMID(vs(i),0,nd+3)+mul+'!X10!U-'+STRMID(vs(i),nd+6,1)+'!N'
 ENDIF ELSE BEGIN
  IF STRMID(vs(i),nd+4,1) EQ '+' THEN $
;
; Caso de exponente positivo de dos dgitos
;
   vv(i)=STRMID(vs(i),0,nd+3)+mul+'!X10!U'+STRMID(vs(i),nd+5,2)+'!N' ELSE $
;
; Caso de exponente negativo de dos dgitos
;
   vv(i)=STRMID(vs(i),0,nd+3)+mul+'!X10!U'+STRMID(vs(i),nd+4,3)+'!N'
 ENDELSE
 IF FLOAT(vs(i)) EQ 0.0 THEN vv(i)='0'
ENDFOR
;
; El primer carcter debe ser un espacio o un signo -. El espacio se elimina
; con STRTRIM (a menos que pongamos NOTRIM). Antes, cambiamos puntos por comas 
; si queremos formato espaol.
; 
IF KEYWORD_SET(SPANISH) THEN vv = CHPOINTCOMMA(vv)
IF KEYWORD_SET(NOTRIM) THEN RETURN, vv ELSE RETURN, STRTRIM(vv,1)

END
