;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; GRPOS v. 3.1, 11 May 2013.                                                   ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure calculates the size and position of the plot (xc, yc, xsize,  ;
;  ysize tags in PPLOT plus !P.POSITION), it optionally modifies               ;
;  pplot.charsize plus !P.CHARSIZE (global character size), and adds           ;
;  pplot.unitl (scalar that converts from cm to normal units for               ;
;  pplot.charsize = 0.8 and which scales with font size) and extraxl, extraxu, ;
;  extrayl, and extrayu (spaces in normal units from the axes to the borders.  ;
;                                                                              ;
; Positional parameters:                                                       ;
; pplot:    Plot structure.                                                    ;
; xxl       Lower x-axis structure.                                            ;
; yyl       Lower y-axis structure.                                            ;
; xxu       Upper x-axis structure.                                            ;
; yyu       Upper y-axis structure.                                            ;
;                                                                              ;
; Version history:                                                             ;
; v1.0:     General clean up.                                                  ;
; v1.1:     New options for DAXIS added.                                       ;
; v2.0:     English version.                                                   ;
; v2.0:     IDLdoc documentation.                                              ;
;           NPX added.                                                         ;
; v2.5:     Version system update.                                             ;
; v3.0:     Adapted to SINGLEPLOT and SINGLECONTOUR changes.                   ;
;           AXISF eliminated and charsize used for input/output.               ;
;           NPX changed into NGX.                                              ;
; v3.1:     Only version change.                                               ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO GRPOS, pplot, xxl, yyl, xxu, yyu, QUIET=quiet
ON_ERROR, 1
IF KEYWORD_SET(QUIET) EQ 0 THEN quiet = 0
IF pplot.xc NE 0.0 AND pplot.xsize EQ 0.0 THEN MESSAGE, 'If you fix plot.xc you should also fix plot.xsize'
IF pplot.yc NE 0.0 AND pplot.ysize EQ 0.0 THEN MESSAGE, 'If you fix plot.yc you should also fix plot.ysize'
IF pplot.charsize EQ 0 THEN BEGIN ; Al principio todos las dimensiones normalizadas suponen pplot.box=[0,0,1,1]
 fixedcharsize = 0
 IF pplot.xsize NE 0.0 THEN BEGIN
  pplot.charsize = pplot.xsize*(pplot.psxsize/19.0)
 ENDIF ELSE BEGIN
  pplot.charsize =         0.8*(pplot.psxsize/19.0)
 ENDELSE
; pplot.charsize  = pplot.charsize*(1.+0.60*(pplot.ngx-1))
ENDIF ELSE BEGIN
 fixedcharsize = 1
ENDELSE
psaspect_eff = pplot.psaspect*(pplot.box[3]-pplot.box[1])/(pplot.box[2]-pplot.box[0])
COPY_STRUC, pplot, pplot, EXCEPT_TAGS='unitl', $
 STRUC_ADD={unitl:1.25*pplot.charsize/pplot.psxsize}
COPY_STRUC, pplot, pplot, EXCEPT_TAGS='extraxl', $
 STRUC_ADD={extraxl:pplot.unitl*((0.25*(yyl.length+0.5)                                              + $                     ; Texto marcas
                                  0.60*(yyl.style EQ 1 AND yyl.title NE ''))*MAX([yyl.style,0])      + $                     ; Ttulo eje y
                             MAX([0.40*(yyl.style EQ 0 OR  yyl.style EQ -1),                           $                     ; Mnimo
			          0.12*xxl.length-0.25*yyl.length,0.12*xxu.length-0.25*yyl.length]))}                        ; Marcas eje x
COPY_STRUC, pplot, pplot, EXCEPT_TAGS='extraxu', $
 STRUC_ADD={extraxu:pplot.unitl*((0.25*(yyu.length+0.5)                                              + $
                                  0.60*(yyu.style EQ 1 AND yyu.title NE ''))*MAX([yyu.style,0])      + $
                             MAX([0.40*(yyu.style EQ 0 OR  yyu.style EQ -1),                           $
			          0.12*xxl.length-0.25*yyu.length,0.12*xxu.length-0.25*yyu.length]))}
COPY_STRUC, pplot, pplot, EXCEPT_TAGS='extrayl', $
 STRUC_ADD={extrayl:pplot.unitl*((0.60                                                               + $
                                  0.60*(xxl.style EQ 1 AND xxl.title NE ''))*MAX([xxl.style,0])      + $
                             MAX([0.40*(xxl.style EQ 0 OR  xxl.style EQ -1),                           $
			          0.05*((yyl.length NE 0 OR yyu.length NE 0) AND xxl.length NE 0)]))/psaspect_eff}
COPY_STRUC, pplot, pplot, EXCEPT_TAGS='extrayu', $
 STRUC_ADD={extrayu:pplot.unitl*((0.60                                                               + $
                                  0.60*(xxu.style EQ 1 AND xxu.title NE ''))*MAX([xxu.style,0])      + $
                             MAX([0.40*(xxu.style EQ 0 OR  xxu.style EQ -1),                           $
			          0.05*((yyl.length NE 0 OR yyu.length NE 0) AND xxu.length NE 0)])  + $
			          0.90*(pplot.title NE '')                                          )/psaspect_eff}
IF fixedcharsize EQ 1 THEN BEGIN
 pplot.extraxl = pplot.extraxl/(pplot.box[2]-pplot.box[0])^0.75
 pplot.extraxu = pplot.extraxu/(pplot.box[2]-pplot.box[0])^0.75
 pplot.extrayl = pplot.extrayl/(pplot.box[3]-pplot.box[1])^0.75
 pplot.extrayu = pplot.extrayu/(pplot.box[3]-pplot.box[1])^0.75
ENDIF
extraxt = pplot.extraxl+pplot.extraxu
extrayt = pplot.extrayl+pplot.extrayu
IF extraxt GT 0.7 OR extrayt GT 0.7 THEN MESSAGE, 'charsize too large!'
IF pplot.xsize EQ 0.0 THEN pplot.xsize = 1.0-extraxt ; xsize not fixed from outside
IF pplot.ysize EQ 0.0 THEN BEGIN                     ; ysize not fixed from outside
 IF extrayt LE extraxt THEN BEGIN
  pplot.ysize    = pplot.xsize
  IF extrayt  NE 0 THEN BEGIN
   pplot.extrayl = pplot.extrayl*extraxt/extrayt
   pplot.extrayu = pplot.extrayu*extraxt/extrayt
  ENDIF ELSE IF pplot.extrayl+pplot.ysize+pplot.extrayu NE 1.0 THEN BEGIN
   pplot.extrayl = 0.5*(1.0-pplot.ysize)
   pplot.extrayu = 0.5*(1.0-pplot.ysize)
  ENDIF
 ENDIF ELSE BEGIN
  pplot.ysize    = 1.0-extrayt
  pplot.xsize    = pplot.ysize
  IF extraxt  NE 0 THEN BEGIN
   pplot.extraxl = pplot.extraxl*extrayt/extraxt
   pplot.extraxu = pplot.extraxu*extrayt/extraxt
  ENDIF ELSE IF pplot.extraxl+pplot.xsize+pplot.extraxu NE 1.0 THEN BEGIN
   pplot.extraxl = 0.5*(1.0-pplot.xsize)
   pplot.extraxu = 0.5*(1.0-pplot.xsize)
  ENDIF
 ENDELSE
ENDIF
extraxt = pplot.extraxl+pplot.extraxu
extrayt = pplot.extrayl+pplot.extrayu
IF (pplot.xsize GT 1.0-extraxt) AND quiet LT 1 THEN PRINT, 'Warning: x size too large: ', pplot.xsize, 1.0-extraxt
IF (pplot.ysize GT 1.0-extrayt) AND quiet LT 1 THEN PRINT, 'Warning: y size too large: ', pplot.ysize, 1.0-extrayt
IF  pplot.xsize GT 1.0 THEN        MESSAGE, 'plot.xsize too large' 
IF  pplot.xsize GT 1.0 THEN        MESSAGE, 'plot.ysize too large'
IF pplot.xc EQ 0.0 THEN BEGIN                       ; xc not fixed from outside
 IF ABS(pplot.xsize+extraxt-1.0) GE 1.0e-7 THEN BEGIN
  pplot.extraxl  = pplot.extraxl*(1.0-pplot.xsize)/extraxt 
  pplot.extraxu  = pplot.extraxu*(1.0-pplot.xsize)/extraxt 
 ENDIF
 pplot.xc        = 0.5+(pplot.extraxl-pplot.extraxu)/2.0
 pplot.xc        = pplot.box[0] + pplot.xc*(pplot.box[2]-pplot.box[0])        ; Renormalizacin
 pplot.xsize     =             pplot.xsize*(pplot.box[2]-pplot.box[0]) 
 pplot.extraxl   =           pplot.extraxl*(pplot.box[2]-pplot.box[0])
 pplot.extraxu   =           pplot.extraxu*(pplot.box[2]-pplot.box[0])
ENDIF ELSE BEGIN                                    ; xc fixed from outside
 pplot.extraxl   =                (pplot.xc - 0.5*pplot.xsize) - pplot.box[0] ; Renormalizacin
 pplot.extraxu   = pplot.box[2] - (pplot.xc + 0.5*pplot.xsize)
ENDELSE
IF pplot.yc EQ 0.0 THEN BEGIN                       ; yc not fixed from outside
 IF ABS(pplot.ysize+extrayt-1.0) GE 1.0e-7 THEN BEGIN
  pplot.extrayl  = pplot.extrayl*(1.0-pplot.ysize)/extrayt 
  pplot.extrayu  = pplot.extrayu*(1.0-pplot.ysize)/extrayt 
 ENDIF
 pplot.yc        = 0.5+(pplot.extrayl-pplot.extrayu)/2.0
 pplot.yc        = pplot.box[1] + pplot.yc*(pplot.box[3]-pplot.box[1])        ; Renormalizacin
 pplot.ysize     =             pplot.ysize*(pplot.box[3]-pplot.box[1])
 pplot.extrayl   =           pplot.extrayl*(pplot.box[3]-pplot.box[1])
 pplot.extrayu   =           pplot.extrayu*(pplot.box[3]-pplot.box[1])
ENDIF ELSE BEGIN                                    ; yc fixed from outside
 pplot.extrayl   =                (pplot.yc - 0.5*pplot.ysize) - pplot.box[1] ; Renormalizacin
 pplot.extrayu   = pplot.box[3] - (pplot.yc + 0.5*pplot.ysize)
ENDELSE
IF fixedcharsize EQ 0 THEN BEGIN
 pplot.charsize = MAX([pplot.charsize*(pplot.box[2]-pplot.box[0]),0.3])       ; Renormalizacin
 pplot.unitl    = 1.25*pplot.charsize/pplot.psxsize
ENDIF
END
