;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; MINMAX2 v. 3.1, 11 May 2013.                                                 ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedures finds the minimum of v1 and the maximum of v2 and optionally ;
;  expands theinterval betwen them.                                            ;
;                                                                              ;
; Positional parameters:                                                       ;
; v1:     Array to use for the minimum.                                        ;
; v2:     Array to use for the maximum.                                        ;
; vmin:   Minimum (output variable).                                           ;
; vmax:   Maximum (output variable).                                           ;
; dv:     Output interval.                                                     ;
;                                                                              ;
; Keyword parameters:                                                          ;
; EXTRAV: Fraction to increase the interval.                                   ;
; LOG:    Flag to use if the scale is logarithmic.                             ;
;                                                                              ;
; Version history:                                                             ;
; v1.0:   General clean up.                                                    ;
; v1.1:   Name changed from MINMAX to MINMAX2 (IDL 5.4 name conflict).         ;
; v2.0:   English version.                                                     ;
; v2.1:   IDLdoc documentation.                                                ;
; v2.5:   Version system update.                                               ;
; v3.0:   Adaptation to global changes in SINGLEPLOT and SINGLECONTOUR.        ;
; v3.1:   Only version change.                                                 ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO MINMAX2, v1, v2, vmin, vmax, dv, EXTRAV=extrav, LOG=log
ON_ERROR, 1
vmin=1.*MIN(v1)
vmax=1.*MAX(v2)
dv = vmax - vmin 
IF KEYWORD_SET(EXTRAV) THEN BEGIN
 IF KEYWORD_SET(LOG) THEN BEGIN
  qv = ALOG10(1.*vmax/vmin)
  vmin = vmin*10^(-0.5*extrav*qv)
  vmax = vmax*10^(+0.5*extrav*qv)
 ENDIF ELSE BEGIN
  vmin = vmin - 0.5*extrav*dv
  vmax = vmax + 0.5*extrav*dv
 ENDELSE
 dv = vmax - vmin 
ENDIF

END
