;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; OPENPS v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure opens a Postscript file.                                      ;
;                                                                              ;
; Positional parameters:                                                       ;
; psname:   Postscript file name.                                              ;
; plot:     Plot structure.                                                    ;
; cfile     Color table structure.                                             ;
;                                                                              ;
; Keyword parameters:                                                          ;
; QUIET:    Flag to supress color table informational messages.                ;
;                                                                              ;
; Version history:                                                             ;
; v1.0:     General clean up.                                                  ;
; v1.1:     Default postcript font changed to Helvetica.                       ;
; v2.0:     English version.                                                   ;
; v2.1:     IDLdoc documentation.                                              ;
; v2.5:     Version system update.                                             ;
;           QUIET flag added.                                                  ;
; v3.0:     Complete remodeling to incorporate structures                      ;
; v3.1:     Font reasignment for compatibility with textoidl                   ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO OPENPS, psname, plot, cfile, QUIET=quiet
ON_ERROR, 1
;
; Ajustamos los parmetros por defecto
;
IF N_ELEMENTS(psname)        EQ 0 OR N_ELEMENTS(plot) EQ 0 THEN $
 MESSAGE, 'OPENPS needs at least two parameters'
IF TAG_EXIST(plot,'landscape') EQ 0 THEN MESSAGE, 'landscape TAG missing in plot structure'
IF TAG_EXIST(plot,'psxoffset') EQ 0 THEN MESSAGE, 'psxoffset TAG missing in plot structure'
IF TAG_EXIST(plot,'psyoffset') EQ 0 THEN MESSAGE, 'psyoffset TAG missing in plot structure'
IF TAG_EXIST(plot,'psxsize')   EQ 0 THEN MESSAGE,   'psxsize TAG missing in plot structure'
IF TAG_EXIST(plot,'psysize')   EQ 0 THEN MESSAGE,   'psysize TAG missing in plot structure'
IF TAG_EXIST(plot,'cmyk')      EQ 0 THEN MESSAGE,      'cmyk TAG missing in plot structure'
IF TAG_EXIST(plot,'portrait')  EQ 0 THEN plot = CREATE_STRUCT(plot, {portrait:1-landscape})
IF N_ELEMENTS(cfile) EQ 0 THEN $
 cfile = {name:'colors1.tbl', dir:!DIR + '/resource/colors/', table:0, def:0}
IF plot.cmyk EQ 1 AND !version.release LT '6.1' THEN MESSAGE, $
 'Sorry, your IDL version is ' + !version.release + ' and CMYK support was not implemented until 6.1'
;
; Abrimos el fichero
;
DEFSYSV, '!DD', !D
SET_PLOT, 'PS'
LOADCT, cfile.table, FILE=cfile.dir+cfile.name, SILENT=quiet
IF !version.release GE '6.1' THEN $
DEVICE, FILENAME=psname, /COLOR, BITS=8, /HELVETICA,         $
	    CMYK=plot.cmyk,                                  $
	PORTRAIT=plot.portrait,  LANDSCAPE=plot.landscape,   $
         XOFFSET=plot.psxoffset,   YOFFSET=plot.psyoffset,   $
	   XSIZE=plot.psxsize,       YSIZE=plot.psysize ELSE $
DEVICE, FILENAME=psname, /COLOR, BITS=8, /HELVETICA,         $
	PORTRAIT=plot.portrait,  LANDSCAPE=plot.landscape,   $
         XOFFSET=plot.psxoffset,   YOFFSET=plot.psyoffset,   $
	   XSIZE=plot.psxsize,       YSIZE=plot.psysize
DEVICE,    /HELVETICA,          FONT_INDEX=3 , ISOLATIN1=1
DEVICE,       /SYMBOL,          FONT_INDEX=4 , ISOLATIN1=1
DEVICE,        /TIMES, /ITALIC, FONT_INDEX=5 , ISOLATIN1=1
DEVICE,    /HELVETICA,   /BOLD, FONT_INDEX=6 , ISOLATIN1=1
DEVICE,        /TIMES,          FONT_INDEX=7 , ISOLATIN1=1
DEVICE,      /COURIER,          FONT_INDEX=8 , ISOLATIN1=1
DEVICE,       /SYMBOL,          FONT_INDEX=9 , ISOLATIN1=1
DEVICE, /ZAPFDINGBATS,          FONT_INDEX=10, ISOLATIN1=1
ERASE
END
