;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; PLOTERR2 v. 3.1, 11 May 2013.                                                ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure draws error bars of the followwing types:                     ;
; x, y+-sy (or x, y+syp-sym)                                                   ;
; x+-sx, y (or x+sxp-sxm, y)                                                   ;
; x+-sx, y+-sy (or x+sxp-sxm, y+syp-sym)                                       ;
; y+syp-sym with an arbitrary angle (in CCW degrees from the top)              ;
;                                                                              ;
; Positional parameters:                                                       ;
; v:        Data structure.                                                    ;
;                                                                              ;
; Version history:                                                             ;
; v1.0:     General clean up.                                                  ;
; v1.1:     SYANGPM added.                                                     ;
; v2.0:     English version.                                                   ;
; v2.1:     IDLdoc documentation.                                              ;
; v2.2:     Bug in angle plots fixed.                                          ;
; v2.5:     Version system update.                                             ;
; v3.0:     Adapted to structures and other issues.                            ;
; v3.1:     Only version change.                                               ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO PLOTERR2, v
ON_ERROR, 1
;
; Inicializamos variables.
;
xrange   = !X.RANGE
yrange   = !Y.RANGE
sgx      = 1*(xrange[1] GT xrange[0]) - 1*(xrange[1] LT xrange[0])
sgy      = 1*(yrange[1] GT yrange[0]) - 1*(yrange[1] LT yrange[0])
xbarsize = (!P.POSITION(2)-!P.POSITION(0))/50.0
ybarsize = (!P.POSITION(3)-!P.POSITION(1))/50.0
barsize  = 0.5*(xbarsize+ybarsize)
;
; Bucle principal
;
FOR i=0,v.n-1 DO BEGIN
;
; Barras de error en x
;
 IF v.ert EQ 1 OR v.ert EQ 3 THEN BEGIN
  xleft     = v.x[i]-v.sxm[i]         
  xright    = v.x[i]+v.sxp[i]         
  aux       = CONVERT_COORD([xleft,xright],[v.y[i],v.y[i]],/DATA,/TO_NORMAL)
  bar1      = CONVERT_COORD(aux[0,0]                    , $
                            aux[1,0]+0.5*xbarsize*[-1,1], /NORMAL,/TO_DATA)
  bar2      = CONVERT_COORD(aux[0,1]                    , $
                            aux[1,1]+0.5*xbarsize*[-1,1], /NORMAL,/TO_DATA)
  OPLOT, [xleft ,xright], [v.y[i],v.y[i]], COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar1[0,*],       bar1[1,*],       COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar2[0,*],       bar2[1,*],       COLOR=v.color[i], THICK=v.thick[i]
 ENDIF
;
; Barras de error en y
;
 IF v.ert EQ 2 OR v.ert EQ 3 THEN BEGIN
  ydown     = v.y[i]-v.sym[i]
  yup       = v.y[i]+v.syp[i]
  aux       = CONVERT_COORD([v.x[i],v.x[i]],[ydown,yup],/DATA,/TO_NORMAL)
  bar1      = CONVERT_COORD(aux[0,0]+0.5*ybarsize*[-1,1], $
                            aux[1,0]                   , /NORMAL,/TO_DATA)
  bar2      = CONVERT_COORD(aux[0,1]+0.5*ybarsize*[-1,1], $
                            aux[1,1]                   , /NORMAL,/TO_DATA)
  OPLOT, [v.x[i], v.x[i]], [ydown,yup  ], COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar1[0,*],        bar1[1,*],     COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar2[0,*],        bar2[1,*],     COLOR=v.color[i], THICK=v.thick[i]
 ENDIF
;
; Barras de error con ngulo. 
;
 IF v.ert GE 4 OR v.ert EQ 5 THEN BEGIN
  ym   =  v.y[i]-v.sym[i]*COS(v.ang[i]*!PI/180.)
  xm   =  v.x[i]+v.sym[i]*SIN(v.ang[i]*!PI/180.)
  yp   =  v.y[i]+v.syp[i]*COS(v.ang[i]*!PI/180.)
  xp   =  v.x[i]-v.syp[i]*SIN(v.ang[i]*!PI/180.)
  aux  = CONVERT_COORD([xm,xp],[ym,yp],/DATA,/TO_NORMAL)
  ang  = ATAN(aux[0,0]-aux[0,1],aux[1,1]-aux[1,0])
  bar1 = CONVERT_COORD(aux[0,0]+0.5*barsize*COS(ang)*[-1,1], $
                       aux[1,0]+0.5*barsize*SIN(ang)*[-1,1], /NORMAL,/TO_DATA)
  bar2 = CONVERT_COORD(aux[0,1]+0.5*barsize*COS(ang)*[-1,1], $
                       aux[1,1]+0.5*barsize*SIN(ang)*[-1,1], /NORMAL,/TO_DATA)
  OPLOT, [xm,xp],   [ym,yp],   COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar1[0,*], bar1[1,*], COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar2[0,*], bar2[1,*], COLOR=v.color[i], THICK=v.thick[i]
 ENDIF
 IF v.ert EQ 5 THEN BEGIN 
  ym   =  v.y[i]-v.sxm[i]*SIN(v.ang[i]*!PI/180.)
  xm   =  v.x[i]-v.sxm[i]*COS(v.ang[i]*!PI/180.)
  yp   =  v.y[i]+v.sxp[i]*SIN(v.ang[i]*!PI/180.)
  xp   =  v.x[i]+v.sxp[i]*COS(v.ang[i]*!PI/180.)
  aux  = CONVERT_COORD([xm,xp],[ym,yp],/DATA,/TO_NORMAL)
  ang  = ATAN(aux[0,0]-aux[0,1],aux[1,1]-aux[1,0])
  bar1 = CONVERT_COORD(aux[0,0]+0.5*barsize*COS(ang)*[-1,1], $
                       aux[1,0]+0.5*barsize*SIN(ang)*[-1,1], /NORMAL,/TO_DATA)
  bar2 = CONVERT_COORD(aux[0,1]+0.5*barsize*COS(ang)*[-1,1], $
                       aux[1,1]+0.5*barsize*SIN(ang)*[-1,1], /NORMAL,/TO_DATA)
  OPLOT, [xm,xp],   [ym,yp],   COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar1[0,*], bar1[1,*], COLOR=v.color[i], THICK=v.thick[i]
  OPLOT, bar2[0,*], bar2[1,*], COLOR=v.color[i], THICK=v.thick[i]
 ENDIF
ENDFOR
END

