;+
; @hidden_file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                              ;
; TICKS2 v. 3.1, 11 May 2013.                                                  ;
; Jess Maz Apellniz, IAA                                                    ;
;                                                                              ;
; This procedure establishes the ticks for an axis and writes the values for   ;
;  tickv, tickname2, minorv, and length.                                       ;
;                                                                              ;
; Positional parameters:                                                       ;
; v:      Axis structure.                                                      ;
; v1:     First critical value of the ratio (for logarithmic axes).            ;
; v2:     Second critical value of the ratio (for logarithmic axes).           ;
; v3:     Third critical value of the ratio (for logarithmic axes).            ;
; v4:     Fourth critical value of the ratio (for logarithmic axes).           ;
;                                                                              ;
; Keyword parameters:                                                          ;
; SPANISH:   Flag to use decimal commas instead of decimal points. only        ;
;             needed if FTICKS is called from here.                            ;
;                                                                              ;
; Version history:                                                             ;
; v0.1:   LESSTICKS introduced.                                                ;
;         A few bugs and nomenclature inconsistencies are fixed.               ;
;         FTICKS is called from here.                                          ;
;         Calculation of length added.                                         ;
; v0.2:   MORETICKS introduced.                                                ;
; v1.0:   General clean up.                                                    ;
; v2.0:   English version.                                                     ;
; v2.1:   IDLdoc documentation.                                                ;
; v2.5:   Version system update.                                               ;
; v3.0    General adaptation to changes in SINGLEPLOT and SINGLECONTOUR.       ;
;         Complete reorganization of input/output                              ;
; v3.1    Minor fix to vminor in logarithmic case.                             ;
;                                                                              ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
PRO TICKS2, v, v1, v2, v3, v4, SPANISH=spanish
ON_ERROR, 1
;
; Procesamos el valor del eje
;
vtickv    = FLTARR(60)
vminoraux = FLTARR(600)
;
; Calculamos el mnimo y mximo redondeados para un eje lineal
;
d     = v.delta
ld    = ALOG10(d) 
nld   = FLOOR(ld) 
n10ld = 10^DOUBLE(nld) 
dd    = d / n10ld 
IF v.lessticks NE 0 THEN BEGIN
 IF  dd GE 5.0 THEN ddd = 2.00*n10ld ELSE $
  IF dd GE 2.5 THEN ddd = 1.00*n10ld ELSE ddd = 0.50*n10ld
ENDIF ELSE IF v.moreticks NE 0 THEN BEGIN
 IF  dd GE 5.0 THEN ddd = 0.50*n10ld ELSE $
  IF dd GE 2.5 THEN ddd = 0.25*n10ld ELSE ddd = 0.10*n10ld
ENDIF ELSE BEGIN
 IF  dd GE 5.0 THEN ddd = 1.00*n10ld ELSE $
  IF dd GE 2.5 THEN ddd = 0.50*n10ld ELSE ddd = 0.25*n10ld
ENDELSE
vvmin = ddd* CEIL(v.range2[0]/ddd)
vvmax = ddd*FLOOR(v.range2[1]/ddd+0.00001)
;
; Caso de eje logartmico
;
IF KEYWORD_SET(v.log) THEN BEGIN
 quot = v.range2[1]/v.range2[0]
 IF KEYWORD_SET(v.lessticks) THEN quot = 2.0*quot
 IF KEYWORD_SET(v.moreticks) THEN quot = 0.5*quot
;
; Para quot menor que v1 se dibujan unas marcas similares a las lineales con
; cinco marcas menores. 
;
 IF quot LE v1 THEN BEGIN
  vticks    = ROUND((vvmax-vvmin)/ddd)
  vtickv    = vvmin + ddd*FINDGEN(vticks+1)
  ddminor   = ddd/5.0
  vminormin = vvmin - ddd
  REPEAT vminormin = vminormin + ddminor UNTIL vminormin GE v.range2[0]
  vminormax = vvmax + ddd
  REPEAT vminormax = vminormax - ddminor UNTIL vminormax LE v.range2[1]
  nminor    = (vminormax-vminormin)/ddminor + 1
  vminor    = vminormin + ddminor*FINDGEN(nminor)
;
; Para quot entre v1 y v2 se dibujan del 1 al 10 con marcas menores en 
; 0,5. Nota: Aqu las marcas menores no incluyen a las mayores, algo que 
; s sucede en los dems casos.
;
 ENDIF ELSE IF quot LE v2 THEN BEGIN
; Marcas principales
  LOGTICKSAUX, v.range2[0], v.range2[1], expl, exph, ddl, ddh, vvminaux, vvmaxaux, $
         explalt, exphalt, ddlalt, ddhalt, vvminauxalt, vvmaxauxalt, $
         vvmin, vvmax, vticks, vtickv
; Marcas menores
  IF (vvminauxalt-0.5)*ddlalt GE v.range2[0] THEN vminorminextra = 1 ELSE $
                                                  vminorminextra = 0
  IF (vvmaxaux   +0.5)*ddh    LE v.range2[1] THEN vminormaxextra = 1 ELSE $
                                                  vminormaxextra = 0
  nminor = (10-vvminaux)+(vvmaxaux-1)+9*(exph-expl-1)+ $
           vminorminextra + vminormaxextra
  vminor = FLTARR(nminor)
  IF vminorminextra EQ 1 THEN vminor[0] = (vvminauxalt-0.5)*ddlalt
  IF vminormaxextra EQ 1 THEN vminor[nminor-1] = (vvmaxaux+0.5)*ddh
  count = vminorminextra
  FOR i=expl,exphalt DO BEGIN
   IF i EQ expl    THEN jmin=vvminaux      ELSE jmin=1
   IF i EQ exphalt THEN jmax=vvmaxauxalt-1 ELSE jmax=9
   FOR j=jmin,jmax DO BEGIN
    vminor[count] = (j+0.5)*10^FLOAT(i)
    count = count + 1
   ENDFOR
  ENDFOR
  IF count NE nminor-vminormaxextra THEN MESSAGE, 'Error in TICKS2'
;
; Para quot entre v2 y v3 se dibujan los casos 1, 2 y 5
;
 ENDIF ELSE IF quot LE v3 THEN BEGIN
; Marcas principales
  ddd = FLOOR(ALOG10(v.range2[0]))
  vminaux = v.range2[0]/10^FLOAT(ddd)
  cont = 0
  IF vminaux EQ 1.0 THEN BEGIN
   type = 10
   vtickv[0] = 10^FLOAT(ddd)
  ENDIF ELSE BEGIN
   IF vminaux LE 2.0 THEN type = 2 ELSE $
   IF vminaux LE 5.0 THEN type = 5 ELSE type = 10
   vtickv[0] = type*10^FLOAT(ddd)
  ENDELSE
  JUMP1: cont = cont + 1
  IF type EQ 2 THEN BEGIN 
   type2 = 5 
   vtickv[cont] = 2.5*vtickv[cont-1]
  ENDIF ELSE IF type EQ 5 THEN BEGIN
   type2 = 10 
   vtickv[cont] = 2.0*vtickv[cont-1]
  ENDIF ELSE BEGIN
   type2 = 2
   vtickv[cont] = 2.0*vtickv[cont-1]
  ENDELSE
  type = type2
  IF vtickv[cont] GT 1.000001*v.range2[1] THEN vticks=cont-1 ELSE GOTO, JUMP1
; Marcas menores
  LOGTICKSAUX, v.range2[0], v.range2[1], expl, exph, ddl, ddh, vvminaux, vvmaxaux, $
         explalt, exphalt, ddlalt, ddhalt, vvminauxalt, vvmaxauxalt, $
         vvmin, vvmax, nminor, vminoraux
   vminor = vminoraux[0:nminor]
;
; Para quot entre v3 y v4 se dibujan los casos 1 y 3
;
 ENDIF ELSE IF quot LE v4 THEN BEGIN
; Marcas principales
  ddd = FLOOR(ALOG10(v.range2[0]))
  vminaux = v.range2[0]/10^FLOAT(ddd)
  cont = 0
  IF vminaux EQ 1.0 THEN BEGIN
   type = 10
   vtickv[0] = 10^FLOAT(ddd)
  ENDIF ELSE BEGIN
   IF vminaux LE 3.0 THEN type = 3 ELSE type = 10
   vtickv[0] = type*10^FLOAT(ddd)
  ENDELSE
  JUMP2: cont = cont + 1
  IF type EQ 3 THEN BEGIN
   type2 = 10
   vtickv[cont] = 10.0*vtickv[cont-1]/3.0
  ENDIF ELSE BEGIN
   type2 = 3
   vtickv[cont] = 3.0*vtickv[cont-1]
  ENDELSE
  type = type2
  IF vtickv[cont] GT 1.000001*v.range2[1] THEN vticks=cont-1 ELSE GOTO, JUMP2
; Marcas menores
  LOGTICKSAUX, v.range2[0], v.range2[1], expl, exph, ddl, ddh, vvminaux, vvmaxaux, $
         explalt, exphalt, ddlalt, ddhalt, vvminauxalt, vvmaxauxalt, $
         vvmin, vvmax, nminor, vminoraux
   vminor = vminoraux[0:nminor]
;
; Para quot mayor que v4 se dibuja poniendo solamente potencias de 10
;
 ENDIF ELSE BEGIN
; Marcas principales
  vminaux =  CEIL(ALOG10(v.range2[0]))
  vmaxaux = FLOOR(ALOG10(v.range2[1]))
  vvmin = 10^FLOAT(vminaux)
  vvmax = 10^FLOAT(vmaxaux)
  IF KEYWORD_SET(LESSTICKS) THEN expch=3.0 ELSE expch=7.0
  expv = ROUND((vmaxaux-vminaux)/expch)
  cont = 0
  vtickv[0] = 10^FLOAT(vminaux)
  JUMP3: cont = cont+1
  vact = vvmin*10^FLOAT(cont*expv)
  IF vact LE 1.000001*vvmax THEN BEGIN
   vtickv[cont] = vact
   GOTO, JUMP3
  ENDIF 
  vticks = cont-1
; Marcas menores
  IF expv EQ 1 THEN BEGIN
   LOGTICKSAUX, v.range2[0], v.range2[1], expl, exph, ddl, ddh, vvminaux, vvmaxaux, $
          explalt, exphalt, ddlalt, ddhalt, vvminauxalt, vvmaxauxalt, $
          vvmin, vvmax, nminor, vminoraux
   vminor = vminoraux[0:nminor]
  ENDIF ELSE BEGIN
   expv = 1
   cont = 0
   vminoraux[0] = 10^FLOAT(vminaux)
   JUMP4: cont = cont+1
   vact = vvmin*10^FLOAT(cont*expv)
   IF vact LE 1.000001*vvmax THEN BEGIN
    vminoraux[cont] = vact
    GOTO, JUMP4
   ENDIF
   nminor = cont
   vminor = vminoraux[0:nminor-1]
  ENDELSE
 ENDELSE
;
; Ajustes finales de las variables
;
; !X.TICKS = vticks
; !X.TICKV = vtickv
 v = CREATE_STRUCT(v, {tickv:vtickv[0:vticks], minorv:vminor})
;
; Caso de eje lineal
;
ENDIF ELSE BEGIN
; !X.TICKS = ROUND((vvmax-vvmin)/ddd)
; !X.TICKV = vvmin + ddd*FINDGEN(!X.TICKS+1)
; vminor = 0
 vticks  = ROUND((vvmax-vvmin)/ddd)
 vticks  = vticks*(vticks GE 0) ; Round-off issues may give negative values in the extreme case of a single point
 vvvmin  = vvmin - ddd
 vminor  = vvvmin + ddd/v.minorn*FINDGEN((vticks+3)*v.minorn+1)
 pos     = WHERE(vminor GE v.range2[0] AND vminor LE v.range2[1], npos)
 IF npos EQ 0 THEN pos = 0      ; Same round-off issues as above
 v = CREATE_STRUCT(v, { tickv:vvmin + ddd*FINDGEN(vticks+1),  $
	               minorv:vminor[pos]                  })
ENDELSE
IF TAG_EXIST(v,'tickname') THEN BEGIN
 v = CREATE_STRUCT(v, {   length:LENGTHV(v.tickname), $
                       tickname2:v.tickname         })
ENDIF ELSE BEGIN
 tickname2 = FTICKS(v.tickv, LOG=v.log, SPANISH=spanish)
 v = CREATE_STRUCT(v, {   length:LENGTHV(tickname2) ,  $
                       tickname2:tickname2          })
ENDELSE
END
